@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# This is a kernel build script for Fedora 15's 2.6.38 kernel.
+# This is a kernel build script for Fedora 13's 2.6.34 kernel.
 #
 
 die () {
@@ -10,17 +10,17 @@ die () {
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
 
-if [ ! -r kernel-2.6.38.8-35.fc15.src.rpm ]
+if [ ! -r kernel-2.6.34.9-69.fc13.src.rpm ]
 then
-    wget http://ftp.riken.jp/Linux/fedora/updates/15/SRPMS/kernel-2.6.38.8-35.fc15.src.rpm || die "Can't download source package."
+    wget http://ftp.riken.jp/Linux/fedora/updates/13/SRPMS/kernel-2.6.34.9-69.fc13.src.rpm || die "Can't download source package."
 fi
-rpm --checksig kernel-2.6.38.8-35.fc15.src.rpm || die "Can't verify signature."
-rpm -ivh kernel-2.6.38.8-35.fc15.src.rpm || die "Can't install source package."
+rpm --checksig kernel-2.6.34.9-69.fc13.src.rpm || die "Can't verify signature."
+rpm -ivh kernel-2.6.34.9-69.fc13.src.rpm || die "Can't install source package."
 
 cd /root/rpmbuild/SOURCES/ || die "Can't chdir to /root/rpmbuild/SOURCES/ ."
-if [ ! -r ccs-patch-1.8.2-20110713.tar.gz ]
+if [ ! -r ccs-patch-1.8.2-20110726.tar.gz ]
 then
-    wget -O ccs-patch-1.8.2-20110713.tar.gz 'http://sourceforge.jp/frs/redir.php?f=/tomoyo/49684/ccs-patch-1.8.2-20110713.tar.gz' || die "Can't download patch."
+    wget -O ccs-patch-1.8.2-20110726.tar.gz 'http://sourceforge.jp/frs/redir.php?f=/tomoyo/49684/ccs-patch-1.8.2-20110726.tar.gz' || die "Can't download patch."
 fi
 
 cd /root/rpmbuild/SPECS/ || die "Can't chdir to /root/rpmbuild/SPECS/ ."
@@ -37,7 +37,7 @@ patch << "EOF" || die "Can't patch spec file."
  ###################################################################
  
  # The buildid can also be specified on the rpmbuild command line
-@@ -427,6 +427,11 @@
+@@ -409,6 +409,11 @@
  # to versions below the minimum
  #
  
@@ -49,7 +49,7 @@ patch << "EOF" || die "Can't patch spec file."
  #
  # First the general kernel 2.6 required versions as per
  # Documentation/Changes
-@@ -486,7 +491,7 @@
+@@ -471,7 +476,7 @@
  AutoProv: yes\
  %{nil}
  
@@ -58,27 +58,27 @@ patch << "EOF" || die "Can't patch spec file."
  Group: System Environment/Kernel
  License: GPLv2
  URL: http://www.kernel.org/
-@@ -902,7 +907,7 @@
+@@ -962,7 +967,7 @@
+ Provides: kernel-devel-uname-r = %{KVERREL}%{?1:.%{1}}\
  AutoReqProv: no\
  Requires(pre): /usr/bin/find\
- Requires: perl\
 -%description -n kernel%{?variant}%{?1:-%{1}}-devel\
 +%description -n ccs-kernel%{?variant}%{?1:-%{1}}-devel\
  This package provides kernel headers and makefiles sufficient to build modules\
  against the %{?2:%{2} }kernel package.\
  %{nil}
-@@ -1469,6 +1474,10 @@
+@@ -1631,6 +1636,10 @@
  
  # END OF PATCH APPLICATIONS
  
 +# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.8.2-20110713.tar.gz
-+patch -sp1 < patches/ccs-patch-2.6.38-fedora-15.diff
++tar -zxf %_sourcedir/ccs-patch-1.8.2-20110726.tar.gz
++patch -sp1 < patches/ccs-patch-2.6.34-fedora-13.diff
 +
  %endif
  
  # Any further pre-build tree manipulations happen here.
-@@ -1497,6 +1506,9 @@
+@@ -1659,6 +1668,9 @@
  for i in *.config
  do
    mv $i .config
@@ -86,8 +86,8 @@ patch << "EOF" || die "Can't patch spec file."
 +  cat config.ccs >> .config
 +  sed -i -e 's:CONFIG_DEBUG_INFO=.*:# CONFIG_DEBUG_INFO is not set:' -- .config
    Arch=`head -1 .config | cut -b 3-`
-   make ARCH=$Arch listnewconfig | grep -E '^CONFIG_' >.newoptions || true
- %if %{listnewconfig_fail}
+   make ARCH=$Arch %{oldconfig_target} > /dev/null
+   echo "# $Arch" > configs/$i
 EOF
 echo ""
 echo ""
@@ -99,5 +99,18 @@ echo ""
 ARCH=`uname -m`
 echo "I'll start 'rpmbuild -bb --target $ARCH --with baseonly --without debug --without debuginfo /root/rpmbuild/SPECS/ccs-kernel.spec' in 30 seconds. Press Ctrl-C to stop."
 sleep 30
+patch << "EOF" || die "Can't patch spec file."
+--- /root/rpmbuild/SPECS/ccs-kernel.spec
++++ /root/rpmbuild/SPECS/ccs-kernel.spec
+@@ -221,7 +221,7 @@
+ 
+ # kernel-PAE is only built on i686.
+ %ifarch i686
+-%define with_pae 1
++%define with_pae 0
+ %else
+ %define with_pae 0
+ %endif
+EOF
 exec rpmbuild -bb --target $ARCH --with baseonly --without debug --without debuginfo /root/rpmbuild/SPECS/ccs-kernel.spec
 exit 0
