@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.1   2011/04/01
+ * Version: 1.8.2   2011/06/20
  */
 
 #include "internal.h"
@@ -37,45 +37,50 @@ static bool ccs_same_reserved(const struct ccs_acl_head *a,
 {
 	const struct ccs_reserved *p1 = container_of(a, typeof(*p1), head);
 	const struct ccs_reserved *p2 = container_of(b, typeof(*p2), head);
-	return p1->min_port == p2->min_port && p1->max_port == p2->max_port;
+	return ccs_same_number_union(&p1->port, &p2->port);
 }
 
 /**
- * ccs_update_reserved_entry - Update "struct ccs_reserved" list.
+ * ccs_write_reserved_port - Update "struct ccs_reserved" list.
  *
- * @min_port:  Start of port number range.
- * @max_port:  End of port number range.
- * @is_delete: True if it is a delete request.
+ * @param: Pointer to "struct ccs_acl_param".
  *
  * Returns 0 on success, negative value otherwise.
  *
  * Caller holds ccs_read_lock().
  */
-static int ccs_update_reserved_entry(const u16 min_port, const u16 max_port,
-				     const bool is_delete)
+int ccs_write_reserved_port(struct ccs_acl_param *param)
 {
-	struct ccs_reserved *ptr;
-	struct ccs_reserved e = {
-		.min_port = min_port,
-		.max_port = max_port
-	};
-	const int error =
-		ccs_update_policy(&e.head, sizeof(e), is_delete,
-				  &ccs_policy_list[CCS_ID_RESERVEDPORT],
-				  ccs_same_reserved);
+	struct ccs_reserved e = { };
+	struct ccs_policy_namespace *ns = param->ns;
+	int error;
 	u8 *tmp;
+	if (param->data[0] == '@' || !ccs_parse_number_union(param, &e.port) ||
+	    e.port.values[1] > 65535 || param->data[0])
+		return -EINVAL;
+	param->list = &ns->policy_list[CCS_ID_RESERVEDPORT];
+	error = ccs_update_policy(&e.head, sizeof(e), param,
+				  ccs_same_reserved);
+	/*
+	 * ccs_put_number_union() is not needed because param->data[0] != '@'.
+	 */
 	if (error)
 		return error;
 	tmp = kzalloc(sizeof(ccs_reserved_port_map), CCS_GFP_FLAGS);
 	if (!tmp)
 		return -ENOMEM;
-	list_for_each_entry_srcu(ptr, &ccs_policy_list[CCS_ID_RESERVEDPORT],
-				 head.list, &ccs_ss) {
-		unsigned int port;
-		if (ptr->head.is_deleted)
-			continue;
-		for (port = ptr->min_port; port <= ptr->max_port; port++)
-			tmp[port >> 3] |= 1 << (port & 7);
+	list_for_each_entry_srcu(ns, &ccs_namespace_list, namespace_list,
+				 &ccs_ss) {
+		struct ccs_reserved *ptr;
+		struct list_head *list = &ns->policy_list[CCS_ID_RESERVEDPORT];
+		list_for_each_entry_srcu(ptr, list, head.list, &ccs_ss) {
+			unsigned int port;
+			if (ptr->head.is_deleted)
+				continue;
+			for (port = ptr->port.values[0];
+			     port <= ptr->port.values[1]; port++)
+				tmp[port >> 3] |= 1 << (port & 7);
+		}
 	}
 	memmove(ccs_reserved_port_map, tmp, sizeof(ccs_reserved_port_map));
 	kfree(tmp);
@@ -86,31 +91,3 @@ static int ccs_update_reserved_entry(const u16 min_port, const u16 max_port,
 	ccsecurity_ops.lport_reserved = __ccs_lport_reserved;
 	return 0;
 }
-
-/**
- * ccs_write_reserved_port - Write "struct ccs_reserved" list.
- *
- * @data:      String to parse.
- * @is_delete: True if it is a delete request.
- *
- * Returns 0 on success, negative value otherwise.
- */
-int ccs_write_reserved_port(char *data, const bool is_delete)
-{
-	unsigned int from;
-	unsigned int to;
-	if (strchr(data, ' '))
-		goto out;
-	switch (sscanf(data, "%u-%u", &from, &to)) {
-	case 1:
-		to = from;
-		/* fall through */
-	case 2:
-		if (from <= to && to < 65536)
-			return ccs_update_reserved_entry(from, to,
-							 is_delete);
-		break;
-	}
-out:
-	return -EINVAL;
-}
