@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.2+   2011/07/07
+ * Version: 1.8.2+   2011/07/13
  */
 
 #include "internal.h"
@@ -528,7 +528,7 @@ static void ccs_check_profile(void)
 	struct ccs_domain_info *domain;
 	const int idx = ccs_read_lock();
 	ccs_policy_loaded = true;
-	printk(KERN_INFO "CCSecurity: 1.8.2+   2011/07/07\n");
+	printk(KERN_INFO "CCSecurity: 1.8.2+   2011/07/13\n");
 	list_for_each_entry_srcu(domain, &ccs_domain_list, list, &ccs_ss) {
 		const u8 profile = domain->profile;
 		const struct ccs_policy_namespace *ns = domain->ns;
@@ -1167,7 +1167,7 @@ static int ccs_write_domain2(struct ccs_policy_namespace *ns,
 		{ "task ", ccs_write_task },
 	};
 	u8 i;
-	for (i = 0; i < 7; i++) {
+	for (i = 0; i < ARRAY_SIZE(ccs_callback); i++) {
 		if (!ccs_str_starts(&param.data, ccs_callback[i].keyword))
 			continue;
 		return ccs_callback[i].write(&param);
@@ -2567,9 +2567,12 @@ static int ccs_write_stat(struct ccs_io_buffer *head)
 	u8 i;
 	if (ccs_str_starts(&data, "Memory used by "))
 		for (i = 0; i < CCS_MAX_MEMORY_STAT; i++)
-			if (ccs_str_starts(&data, ccs_memory_headers[i]))
+			if (ccs_str_starts(&data, ccs_memory_headers[i])) {
+				if (*data == ' ')
+					data++;
 				ccs_memory_quota[i] =
 					simple_strtoul(data, NULL, 10);
+			}
 	return 0;
 }
 
